@@ -24,6 +24,10 @@ degrees <- function(radian) {
 #' center of Jupiter's disk from the equatorial plane in the units of Jupiter's
 #' equatorial radius; Y is positive toward the north
 #'
+#'   u_corrected - the corrected angular position of the satellite in degrees,
+#' used to determine visibility conditions (whether a moon can be seen against
+#' Jupiter's disk or is hidden behind it)
+#'
 #' @details
 #' The function is based on algorithms in the book:
 #' Astronomical Formulae for Calculators (4th edition), Jean Meeus, Willmann-Bell Inc., 1988
@@ -35,11 +39,12 @@ degrees <- function(radian) {
 #' @param minute Type in the minute (integer number from 0 to 59).
 #'
 #' @returns
-#' `data.frame`: 4 observations of 3 variables:
-#' $ moon: chr "Io" "Europa" "Ganymede" "Callisto"
-#' $ x   : num
-#' $ y   : num
-#' Four rows - each row has the position (x,y) of one moon.
+#' `data.frame`: 4 observations of 4 variables:
+#' $ moon       : chr "Io" "Europa" "Ganymede" "Callisto"
+#' $ x          : num
+#' $ y          : num
+#' $ u_corrected: num
+#' Four rows - each row has the position (x,y) and corrected angular position (u_corrected) of one moon.
 #' Additionally, the positions of the moons are shown graphically.
 #'
 #' @importFrom png readPNG
@@ -48,6 +53,11 @@ degrees <- function(radian) {
 #'
 #' @examples
 #' galsat(2025, 10, 13, 23, 30)
+#' # Also try these interesting configuration moments:
+#' galsat(2021, 8, 15, 15, 48)
+#' galsat(2032, 1, 5, 6, 44)
+#' galsat(2033, 7, 28, 4, 50)
+#' galsat(2039, 7, 31, 18, 55)
 
 galsat <- function(year, month, day, hour, minute) {
 
@@ -172,7 +182,8 @@ galsat <- function(year, month, day, hour, minute) {
     p <- data.frame(
         moon = c("Io", "Europa", "Ganymede", "Callisto"),
         x = c(x1, x2, x3, x4),
-        y = c(y1, y2, y3, y4)
+        y = c(y1, y2, y3, y4),
+        u_corrected = c(u1_corrected, u2_corrected, u3_corrected, u4_corrected)
     )
 
     # inserting a title, date and time
@@ -196,15 +207,163 @@ galsat <- function(year, month, day, hour, minute) {
     }
     if (sqrt(x2^2 + y2^2) > 1 | u2_corrected < 90 | u2_corrected > 270) {
         graphics::points(x2, y2, col = "blue", pch = 20);
-        graphics::text(x2, y2 + 3, "E", col = "blue", cex = 0.8, adj = 0.5)
+        graphics::text(x2, y2 + 6, "E", col = "blue", cex = 0.8, adj = 0.5)
     }
     if (sqrt(x3^2 + y3^2) > 1 | u3_corrected < 90 | u3_corrected > 270) {
         graphics::points(x3, y3, col = "green", pch = 20);
-        graphics::text(x3, y3 + 3, "G", col = "green", cex = 0.8, adj = 0.5)
+        graphics::text(x3, y3 + 9, "G", col = "green", cex = 0.8, adj = 0.5)
     }
     if (sqrt(x4^2 + y4^2) > 1 | u4_corrected < 90 | u4_corrected > 270) {
         graphics::points(x4, y4, col = "magenta", pch = 20);
-        graphics::text(x4, y4 + 3, "C", col = "magenta", cex = 0.8, adj = 0.5)
+        graphics::text(x4, y4 + 12, "C", col = "magenta", cex = 0.8, adj = 0.5)
     }
     return(p)
 }
+
+#' Animate the motion of Jupiter's Galilean satellites
+#'
+#' @description
+#' `galsat_animate()` creates an animation showing the orbital motion of Jupiter's
+#' four largest satellites over time. The function starts from a user-specified
+#' time and advances in regular intervals to show how the moons move around Jupiter.
+#'
+#' @details
+#' The animation uses the galsat() function internally to calculate satellite
+#' positions at each time step. Time is advanced by the specified interval
+#' (default 5 minutes) for each frame of the animation.
+#'
+#' @param year Type in the starting year (integer number from 0 to 3000).
+#' @param month Type in the starting month (integer number from 1 to 12).
+#' @param day Type in the starting day (integer number from 1 to 31).
+#' @param hour Type in the starting hour (integer number from 0 to 23).
+#' @param minute Type in the starting minute (integer number from 0 to 59).
+#' @param duration_hours Duration of the animation in hours (default 24).
+#' @param time_step_minutes Time step between frames in minutes (default 5).
+#' @param pause_seconds Pause between frames in seconds (default 0.05).
+#'
+#' @returns
+#' Creates an animated plot showing the orbital motion of Jupiter's moons.
+#' Returns invisibly the final positions data frame.
+#'
+#' @export
+#'
+#' @examples
+#' # Animate 2 hours with 10-minutes steps
+#' galsat_animate(2025, 10, 6, 21, 50, duration_hours = 2, time_step_minutes = 10)
+
+galsat_animate <- function(year, month, day, hour, minute,
+                           duration_hours = 24,
+                           time_step_minutes = 5,
+                           pause_seconds = 0.05) {
+
+    # Validate inputs
+    if (!is.numeric(c(year, month, day, hour, minute, duration_hours, time_step_minutes, pause_seconds))) {
+        stop("All parameters must be numeric.")
+    }
+
+    if (duration_hours <= 0 || time_step_minutes <= 0) {
+        stop("Duration and time step must be positive values.")
+    }
+
+    if (pause_seconds < 0) {
+        stop("Pause cannot be negative value.")
+    }
+
+    # Calculate total number of frames
+    total_minutes <- duration_hours * 60
+    n_frames <- ceiling(total_minutes / time_step_minutes)
+
+    # Create the animation
+    for (i in 1:n_frames) {
+        # Calculate current time
+        current_minute <- minute + ((i - 1) * time_step_minutes)
+        current_hour <- hour
+        current_day <- day
+        current_month <- month
+        current_year <- year
+
+        # Handle minute overflow
+        if (current_minute >= 60) {
+            extra_hours <- floor(current_minute / 60)
+            current_minute <- current_minute %% 60
+            current_hour <- current_hour + extra_hours
+        }
+
+        # Handle hour overflow
+        if (current_hour >= 24) {
+            extra_days <- floor(current_hour / 24)
+            current_hour <- current_hour %% 24
+            current_day <- current_day + extra_days
+        }
+
+        # Simple day overflow handling (not perfect for all months/years, but sufficient)
+        max_day <- 31
+        if (current_month %in% c(4, 6, 9, 11)) max_day <- 30
+        if (current_month == 2) max_day <- 28
+        if (current_day > max_day) {
+            current_day <- current_day - max_day
+            current_month <- current_month + 1
+            if (current_month > 12) {
+                current_month <- 1
+                current_year <- current_year + 1
+            }
+        }
+
+        # Get satellite positions for current time
+        tryCatch({
+            positions <- galsat(current_year, current_month, current_day, current_hour, current_minute)
+
+            # Clear the plot and redraw
+            graphics::plot(c(-30, 30), c(-30, 30), type = "n", axes = FALSE, xlab = "", ylab = "", asp = 1)
+
+            # Add title with animation info
+            graphics::text(0, 28, "ANIMATION: SATELLITES OF JUPITER", col = "black", cex = 1.7, adj = 0.5)
+            graphics::text(0, 23,
+                           paste0('Date: ', current_year, '-', sprintf("%02d", current_month), '-', sprintf("%02d", current_day)),
+                           col = "black", cex = 1.2, adj = 0.5)
+            graphics::text(0, 18,
+                           paste0('Time [UTC]: ', sprintf("%02d", current_hour), ':', sprintf("%02d", current_minute)),
+                           col = "black", cex = 1.2, adj = 0.5)
+            graphics::text(0, 14,
+                           paste0('Frame: ', i, '/', n_frames, ' (Step: ', time_step_minutes, ' min)'),
+                           col = "gray50", cex = 1.0, adj = 0.5)
+
+            # Draw Jupiter
+            jupiter <- png::readPNG(system.file("jupiter.png", package = "galisats"))
+            graphics::rasterImage(jupiter, xleft = -1, ybottom = -1, xright = 1, ytop = 1)
+
+            # Draw current satellite positions
+            colors <- c("red", "blue", "green", "magenta")
+            labels <- c("I", "E", "G", "C")
+
+            for (j in 1:nrow(positions)) {
+                x_pos <- positions$x[j]
+                y_pos <- positions$y[j]
+                moon_name <- positions$moon[j]
+                u_corrected <- positions$u_corrected[j]
+
+                # Calculates the distance from the center of Jupiter's disk
+                r <- sqrt(x_pos^2 + y_pos^2)
+
+                if (r > 1 || u_corrected < 90 || u_corrected > 270) {  # moon is visible (not behind Jupiter)
+                    # Draw moon
+                    graphics::points(x_pos, y_pos, col = colors[j], pch = 20)
+                    # Draw label
+                    graphics::text(x_pos, y_pos + 3 * j, labels[j], col = colors[j], cex = 0.8, adj = 0.5)
+                }
+            }
+
+            # Add legend
+            graphics::text(0, -18, "I = Io    E = Europa    G = Ganymede    C = Callisto", col = "gray50", cex = 0.8, adj = 0.5)
+
+            # Pause between frames
+            Sys.sleep(pause_seconds)
+
+        }, error = function(e) {
+            warning(paste("Error at frame", i, ":", e$message))
+        })
+    }
+
+    # Return the final positions
+    invisible(positions)
+}
